@@ -4,19 +4,9 @@
 This module provides the main GRL class and utility functions
 """
 
-from .info import USE_LXML, USE_YAML
-
-if USE_LXML:
-    import lxml.etree as ET
-else:
-    import xml.etree.ElementTree as ET
-    import xml.dom.minidom as minidom
-    # minidom toprettyxml replacement
-    from xml.dom.ext import PrettyPrint
-
-if USE_YAML:
-    import yaml
+from . import info
 
+import sys
 import os
 import copy
 import urllib2
@@ -27,6 +17,7 @@ import bisect
 import datetime
 import cStringIO
 import re
+import xml.etree.ElementTree as _ET
 
 
 __all__ = [
@@ -40,6 +31,46 @@ __all__ = [
 ]
 
 
+XML_WHITESPACE = re.compile('>\n\s+([^<>\s].*?)\n\s+</', re.DOTALL)
+LBRANGE_ORDER = re.compile('<LBRange End="(?P<end>\d+)" Start="(?P<start>\d+)"/>')
+
+
+def fix_attr_order(match):
+    return '<LBRange Start="%s" End="%s"/>' % (
+        match.group('start'),
+        match.group('end'))
+
+
+def pretty_xml(doc):
+    """
+    Fix-up minidom's toprettyxml() output
+
+    http://stackoverflow.com/questions/749796/pretty-printing-xml-in-python
+    http://stackoverflow.com/a/3367423/1002176
+    """
+    # remove extra whitespace created by minidom
+    xml = XML_WHITESPACE.sub('>\g<1></', doc.toprettyxml(indent='  '))
+    # preserve order of Start and End attributes (although the order of XML
+    # node attributes is not important and should not be depended on)
+    xml = LBRANGE_ORDER.sub(fix_attr_order, xml)
+    return xml[len('<?xml version="1.0" ?>') + 1:]
+
+
+def get_tree_builder():
+    if info.USE_LXML:
+        import lxml.etree as ET
+    else:
+        import xml.etree.ElementTree as ET
+    class _MetadataTreeBuilder(ET.TreeBuilder):
+        """
+        Implements doctype() as required to avoid deprecation warnings with
+        >=python-2.7.
+        """
+        def doctype(self, name, pubid, system):
+            pass
+    return _MetadataTreeBuilder()
+
+
 def clipped(grl, startrun=None, startlb=None, endrun=None, endlb=None):
     """
     Return a clipped GRL between startrun, startlb and
@@ -63,7 +94,7 @@ def clipped(grl, startrun=None, startlb=None, endrun=None, endlb=None):
 
 def diffed(*args):
     """
-    Return the difference of multiple GRLs: (((A-B)-C)-D)...)
+    Return the difference of multiple GRLs: (((A-B)-C)-D) ...)
 
     *args*: tuple of GRLs
     """
@@ -72,7 +103,7 @@ def diffed(*args):
 
 def ored(*args):
     """
-    Return the OR of multiple GRLs: A | B | C...
+    Return the OR of multiple GRLs: A | B | C ...
 
     *args*: tuple of GRLs
     """
@@ -81,7 +112,7 @@ def ored(*args):
 
 def anded(*args):
     """
-    Return the AND of multiple GRLs: A & B & C...
+    Return the AND of multiple GRLs: A & B & C ...
 
     *args*: tuple of GRLs
     """
@@ -90,7 +121,7 @@ def anded(*args):
 
 def xored(*args):
     """
-    Return the XOR of multiple GRLs: A ^ B ^ C...
+    Return the XOR of multiple GRLs: A ^ B ^ C ...
 
     *args*: tuple of GRLs
     """
@@ -173,7 +204,7 @@ class GRL(object):
         'cut'
     ]
 
-    ROOT_PATTERN = re.compile('\.root(\..+)?:/')
+    ROOT_PATTERN = re.compile(r'\.root[^ \t\n\r\f\v:/]*:/')
 
     def __init__(self, grl=None, from_string=False):
         """
@@ -208,11 +239,11 @@ class GRL(object):
                     # one place where goodruns requires ROOT
                     try:
                         import ROOT
-                        ROOT.PyConfig.IgnoreCommandLineOptions = True
                     except ImportError:
-                        raise ImportError('Specified GRL in ROOT file '
-                                          'but cannot import ROOT. Are ROOT '
-                                          'and PyROOT installed?')
+                        raise ImportError(
+                            'Specified a GRL in a ROOT file '
+                            'but cannot import ROOT. Is ROOT installed '
+                            'with PyROOT enabled?')
                     cwd = ROOT.gDirectory
                     filename, _, path = grl.rpartition(':/')
                     root_file = ROOT.TFile.Open(filename)
@@ -238,10 +269,19 @@ class GRL(object):
                 filename = grl.name
             name, ext = os.path.splitext(filename)
             if filename == "<stdin>" or ext == '.xml':
-                tree = ET.parse(grl)
+                if info.USE_LXML:
+                    import lxml.etree as ET
+                else:
+                    import xml.etree.ElementTree as ET
+                if sys.version_info >= (2, 7):
+                    tree = ET.parse(grl, parser=ET.XMLParser(
+                        target=get_tree_builder()))
+                else:
+                    tree = ET.parse(grl)
                 self.from_xml(tree)
             elif ext == '.yml':
-                if USE_YAML:
+                if info.USE_YAML:
+                    import yaml
                     if isinstance(grl, file):
                         self.from_dict(yaml.load(grl))
                     else:
@@ -265,7 +305,16 @@ class GRL(object):
 
         *string*: str
         """
-        tree = ET.fromstring(string)
+        if info.USE_LXML:
+            import lxml.etree as ET
+        else:
+            import xml.etree.ElementTree as ET
+        if sys.version_info >= (2, 7):
+            tree = ET.XML(string, parser=ET.XMLParser(
+                target=get_tree_builder()))
+        else:
+            tree = ET.fromstring(string)
+
         self.from_xml(tree)
 
     def from_xml(self, tree):
@@ -719,6 +768,11 @@ class GRL(object):
         *format*: str
         """
         if format == 'xml':
+            if info.USE_LXML:
+                import lxml.etree as ET
+            else:
+                import xml.etree.ElementTree as ET
+                import xml.dom.minidom as minidom
             root = ET.Element('LumiRangeCollection')
             subroot = ET.SubElement(root, 'NamedLumiRange')
             name = ET.SubElement(subroot, 'Name')
@@ -742,17 +796,20 @@ class GRL(object):
             '''"http://atlas-runquery.cern.ch/LumiRangeCollection.dtd">\n'''
             '''<!-- This document was created by goodruns: '''
             '''http://pypi.python.org/pypi/goodruns/ on %s -->\n''' % date)
+            filehandle.write('<?xml version="1.0"?>\n')
+            filehandle.write(meta)
             tree = ET.ElementTree(root)
-            if USE_LXML:
-                filehandle.write('<?xml version="1.0"?>\n')
-                filehandle.write(meta)
+            if info.USE_LXML:
                 tree.write(filehandle, pretty_print=True)
             else:
-                # get pretty XML from ElementTree
-                xml = minidom.parseString(meta +
-                                          ET.tostring(tree.getroot(), 'utf-8'))
-                PrettyPrint(xml, stream=filehandle, encoding='utf-8')
+                xml = minidom.parseString(ET.tostring(tree.getroot(), 'utf-8'))
+                filehandle.write(pretty_xml(xml))
         elif format in ('yml', 'yaml'):
+            if not info.USE_YAML:
+                raise RuntimeError(
+                    "goodruns was not installed with YAML support")
+            else:
+                import yaml
             filehandle.write(yaml.dump(self.to_dict()))
         elif format == 'txt':
             filehandle.write(str(self) + '\n')
