@@ -6,10 +6,10 @@
  \__, |\___/ \___/ \__,_|_|   \__,_|_| |_|___/  %s
  |___/
 
- Author: |\|031 ])4vv3
+ Author: Noel Dawe
 
 """
-__version__ = '2.6'
+__version__ = '2.7'
 __url__ = 'http://ndawe.github.com/goodruns'
 __repo_url__ = 'https://github.com/ndawe/goodruns/'
 __download_url__ = 'http://pypi.python.org/packages/source/g/goodruns/' \
